/** @interface */
declare class Validator {
    /**
     * Validates the data. If they are invalid an Error is thrown.
     *
     * @param {any} data The data to validate.
     */
    validate(data: any): void;
}

declare class ValidatorHandler {
    /**
     * Object used to manage the validation through Validator objects.
     *
     * @param {Validator[]} validators Array that contains Validator objects used
     *   to validate data.
     */
    constructor(validators: Validator[]);
    /** @param {any} data Data to validate. */
    validate(data: any): void;
    #private;
}

declare class SerializerHandler {
    /**
     * Object used to manage the Serializer objects.
     *
     * @param {ValidatorHandler} [serializerValidatorHandler] A ValidatorHandler
     *   object used to validate Serializers.
     */
    constructor(serializerValidatorHandler?: ValidatorHandler);
    /**
     * Returns an object that contains the Serializers added to JsonSerializer
     * object. The keys are obtained from Serializer.getSerializerType method and
     * the values are the Serializer objects.
     *
     * @returns {object} Object that contains the Serializers added to
     *   JsonSerializer object.
     */
    getSerializers(): object;
    /**
     * Adds a Serializer. If it is invalid an Error will be thrown.
     *
     * @param {object} serializer An object used to serialize and unserialize
     *   data.
     * @param {() => string} serializer.getSerializerType An function that return
     *   the type of Serializer.
     * @param {(unserializedData: any) => object} serializer.serialize An function
     *   that serialize data.
     * @param {(serializedData: string) => any} serializer.parse An function that
     *   unserialize data.
     */
    addSerializer(serializer: {
        getSerializerType: () => string;
        serialize: (unserializedData: any) => object;
        parse: (serializedData: string) => any;
    }): void;
    #private;
}

declare class ReplacerBuilder {
    /**
     * Sets Serializers. When the build method is called, the Serializers are
     * added to the replacer function.
     *
     * @param {object} serializers An object that contains Serializers added to
     *   JsonSerializer object. The keys are obtained from
     *   Serializer.getSerializerType method and the values are the Serializer
     *   objects.
     */
    setSerializers(serializers: object): void;
    /**
     * Builds a replacer function using builder pattern.
     *
     * @returns {(key: string, value: any) => any} Replacer function used by
     *   JSON.stringify method.
     */
    build(): (key: string, value: any) => any;
    #private;
}

declare class ReviverBuilder {
    /**
     * Sets Serializers. When the build method is called, the Serializers are
     * added to the reviver function.
     *
     * @param {object} serializers An object that contains Serializers added to
     *   JsonSerializer object. The keys are obtained from
     *   Serializer.getSerializerType method and the values are the Serializer
     *   objects.
     */
    setSerializers(serializers: object): void;
    /**
     * Builds a reviver function using builder pattern.
     *
     * @returns {(key: string, value: any) => any} Reviver function used by
     *   JSON.parse method.
     */
    build(): (key: string, value: any) => any;
    #private;
}

/**
 * @callback replacer
 * @param {string} key
 * @param {any} value
 */
/**
 * @callback reviver
 * @param {string} key
 * @param {any} value
 */
declare class JsonSerializer {
    /**
     * JSON wrapper used to extend the functionality of JSON.stringify and
     * JSON.parse methods through Serializers.
     *
     * @param {ReplacerBuilder} replacerBuilder The object used to obtain the
     *   replacer function.
     * @param {ReviverBuilder} reviverBuilder The object used to obtain the
     *   reviver function.
     * @param {SerializerHandler} serializerHandler The object used to manage the
     *   Serializer objects.
     */
    constructor(replacerBuilder: ReplacerBuilder, reviverBuilder: ReviverBuilder, serializerHandler: SerializerHandler);
    /**
     * Returns an object that contains the serializers added to JsonSerializer
     * object. The keys are obtained from serializer.getSerializerType method and
     * the values are the Serializer objects.
     *
     * @returns {object} Object that contains the serializers added to
     *   JsonSerializer object.
     */
    getSerializers(): object;
    /**
     * Adds serializers through the serializersInstaller and to update the
     * JsonSerializer object.
     *
     * @param {object} serializersInstaller An object that adds serializers using
     *   the install method.
     * @param {(
     *   serializerHandler: SerializerHandler,
     *   installOptions: object,
     * ) => void} serializersInstaller.install
     *   The method used to add serializers to JsonSerializer object.
     * @param {object} [installOptions] An object that contains the install
     *   options.
     */
    installSerializersAndRefreshJsonSerializer(serializersInstaller: {
        install: (serializerHandler: SerializerHandler, installOptions: object) => void;
    }, installOptions?: object): void;
    /**
     * Adds a Serializer and to update the JsonSerializer object.
     *
     * @param {object} serializer An object used to serialize and unserialize
     *   data.
     * @param {() => string} serializer.getSerializerType An function that returns
     *   the type of serializer.
     * @param {(unserializedData: any) => object} serializer.serialize An function
     *   that serializes data.
     * @param {(serializedData: string) => any} serializer.parse An function that
     *   unserializes data.
     */
    addSerializerAndRefreshJsonSerializer(serializer: {
        getSerializerType: () => string;
        serialize: (unserializedData: any) => object;
        parse: (serializedData: string) => any;
    }): void;
    /**
     * Transforms a javascript value to a JSON string using JSON.stringify method.
     *
     * According to JSON.stringify's documentation the parameters represent.
     *
     * @param {any} unserializedData A javascript value, usually an object or
     *   array, to be converted.
     * @param {string | number} space Adds indentation, white space, and line
     *   break characters to the return-value JSON text to make it easier to
     *   read.
     * @returns {string} A JSON string generated by JSON.stringify method.
     */
    serialize(unserializedData: any, space: string | number): string;
    /**
     * Transform a JSON string to a javascript value using JSON.parse method.
     *
     * According to JSON.parse's documentation the parameters represent.
     *
     * @param {string} serializedData A valid JSON string.
     * @returns {any} A javascript value generated by JSON.parse method.
     */
    parse(serializedData: string): any;
    #private;
}

declare class JsonSerializerFactory {
    /**
     * Create a new instance of JsonSerializer
     *
     * @returns {JsonSerializer} A new instance of JsonSerializer
     */
    static createJsonSerializer(): JsonSerializer;
}

export { JsonSerializerFactory };
